#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022-present Lenovo
# Confidential and Proprietary
import argparse
import hashlib
import os
import time
from itertools import dropwhile, takewhile

NEW_VERSION = "1.5.0"
LICOTOTEMPLATE = {
    "6.0": "1.0.0",
    "6.1": "1.1.0",
    "6.2": "1.2.0",
    "6.3": "1.3.0",
    "6.4": "1.4.0",
    "6.4.1": "1.4.1",
    "7.0": "1.5.0"
}
OLD_EXPORT_KEY = [
    'version', 'MagicNumber', 'exporter', 'export_time',
    'source', 'scheduler', 'name', 'logo', 'desc',
    'parameters_json', 'template_file'
]
NEW_EXPORT_KEY = [
    'version', 'MagicNumber', 'exporter', 'export_time',
    'source', 'scheduler', 'index', 'name', 'category',
    'logo', 'desc', 'parameters_json', 'template_file'
]


def print_green(string):
    print(f'\033[92m{string}\033[0m')


def md5value(s):
    md5 = hashlib.md5()  # nosec B324
    md5.update(s.encode("utf-8"))
    return md5.hexdigest()


def get_key(version):
    if "1.0.0" <= version < "1.5.0":
        key = OLD_EXPORT_KEY
    elif "1.5.0" <= version <= NEW_VERSION:
        key = NEW_EXPORT_KEY
    else:
        raise Exception("Unsupported job template version.")
    return key


def export_template_file(data, convert_version):
    text = ''
    separator = '=' * 30 + data.get('MagicNumber') + '=' * 30 + "\n"
    exist_separator = False
    try:
        keys = get_key(convert_version)
    except Exception:
        raise
    for key in keys:
        line_content = str(data.get(key)) + '\n'
        if key in ['desc', 'parameters_json', 'template_file']:
            if exist_separator:
                line_content = line_content + separator
            else:
                line_content = separator + line_content + separator
            exist_separator = True

        text += line_content

    Checksum = 'LICO' + text + 'LICO'
    text += md5value(Checksum)
    return text


def get_content_from_file(
        job_file, convert_version, specified_version=''
):
    content_list = job_file.readlines()
    if not content_list:
        raise Exception("File content cannot be empty.")
    if specified_version:
        version = specified_version
    else:
        version_split = content_list[0].strip()
        if not version_split:
            raise Exception("This is not a job template exported by LiCO.")
        version = version_split.decode(encoding='utf8')
    try:
        key = get_key(version)
        return parse_file_contents(content_list, key, convert_version)
    except Exception:
        raise


def parse_file_contents(content_list, key, convert_version):
    import_contents = {
        "index": 99999,
        "category": "General",
        "version": convert_version
    }
    magic_line = b'=' * 30 + \
                 content_list[1].splitlines()[0] + b'=' * 30 + b'\n'
    content_data = list(takewhile(lambda x: x != magic_line, content_list))
    for i in range(len(content_data)):
        if i == 0:
            continue
        import_contents.update(
            {
                key[i]: content_data[i].strip().decode(encoding='utf8')
            }
        )
    magic_data_list = list(dropwhile(lambda x: x != magic_line, content_list))
    magic_data = b"".join(magic_data_list[1:]).split(magic_line)[:-1]
    len_magic_data = len(magic_data)
    for i in range(len_magic_data):
        import_contents.update(
            {
                key[-len_magic_data:][i]: magic_data[i].strip().decode(
                    encoding='utf8'
                )
            }
        )
    return import_contents


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--existed-template', required=True,
        help='Path of private template exported from LiCO 6 series.'
    )
    parser.add_argument(
        '--version', required=False,
        choices=LICOTOTEMPLATE.values()
    )
    parser.add_argument(
        "--lico-version", required=False,
        choices=LICOTOTEMPLATE.keys(),
        help="Specify the LiCO version that needs to "
             "use the converted template file."
    )
    parser.add_argument(
        '--output', required=False,
        help="Path of the new import template file to be output."
    )
    args = parser.parse_args().__dict__

    template = args['existed_template']
    if not os.path.isfile(template):
        print(f"The path {template} does not exist or is not a file.")
        exit(-1)

    output = args.get('output', '')
    if not output:
        output = f"output-{int(time.time())}.ljt"
    if os.path.exists(output):
        print(f"The path {output} already exists.")
        exit(-1)

    specified_version = args.get('version')
    if not specified_version:
        specified_version = ''

    lico_version = args.get('lico_version')
    if not lico_version:
        lico_version = "7.0"
    convert_version = LICOTOTEMPLATE[lico_version]

    with open(template, 'rb') as template_file:
        try:
            content = get_content_from_file(
                template_file, convert_version,
                specified_version
            )
            export_content = export_template_file(
                content, convert_version
            )
            with open(output, 'w') as output_file:
                output_file.write(export_content)
        except Exception as e:
            print(e)
            exit(-1)
    print_green("The conversion was successful.")
